'use strict';

var d3Color = require('d3-color');
var d3Arrays = require('d3-arrays');
var d3Interpolate = require('d3-interpolate');
var d3Format = require('d3-format');
var d3TimeFormat = require('d3-time-format');
var d3Time = require('d3-time');

function nice(domain, step) {
  domain = domain.slice();
  if (!step) return domain;

  var i0 = 0,
      i1 = domain.length - 1,
      x0 = domain[i0],
      x1 = domain[i1],
      t;

  if (x1 < x0) {
    t = i0, i0 = i1, i1 = t;
    t = x0, x0 = x1, x1 = t;
  }

  domain[i0] = Math.floor(x0 / step) * step;
  domain[i1] = Math.ceil(x1 / step) * step;
  return domain;
};

var e10 = Math.sqrt(50);
var e5 = Math.sqrt(10);
var e2 = Math.sqrt(2);
function tickRange(domain, count) {
  if (count == null) count = 10;

  var start = domain[0],
      stop = domain[domain.length - 1];

  if (stop < start) error = stop, stop = start, start = error;

  var span = stop - start,
      step = Math.pow(10, Math.floor(Math.log(span / count) / Math.LN10)),
      error = span / count / step;

  // Filter ticks to get closer to the desired count.
  if (error >= e10) step *= 10;
  else if (error >= e5) step *= 5;
  else if (error >= e2) step *= 2;

  // Round start and stop values to step interval.
  return [
    Math.ceil(start / step) * step,
    Math.floor(stop / step) * step + step / 2, // inclusive
    step
  ];
};

function ticks$1(domain, count) {
  return d3Arrays.range.apply(null, tickRange(domain, count));
};

function tickFormat$2(domain, count, specifier) {
  var range = tickRange(domain, count);
  if (specifier == null) {
    specifier = ",." + d3Format.precisionFixed(range[2]) + "f";
  } else {
    switch (specifier = d3Format.formatSpecifier(specifier), specifier.type) {
      case "s": {
        var value = Math.max(Math.abs(range[0]), Math.abs(range[1]));
        if (specifier.precision == null) specifier.precision = d3Format.precisionPrefix(range[2], value);
        return d3Format.formatPrefix(specifier, value);
      }
      case "":
      case "e":
      case "g":
      case "p":
      case "r": {
        if (specifier.precision == null) specifier.precision = d3Format.precisionRound(range[2], Math.max(Math.abs(range[0]), Math.abs(range[1]))) - (specifier.type === "e");
        break;
      }
      case "f":
      case "%": {
        if (specifier.precision == null) specifier.precision = d3Format.precisionFixed(range[2]) - (specifier.type === "%") * 2;
        break;
      }
    }
  }
  return d3Format.format(specifier);
};

function uninterpolateClamp(a, b) {
  b = (b -= a = +a) || 1 / b;
  return function(x) {
    return Math.max(0, Math.min(1, (x - a) / b));
  };
}

function uninterpolateNumber(a, b) {
  b = (b -= a = +a) || 1 / b;
  return function(x) {
    return (x - a) / b;
  };
}

function bilinear(domain, range, uninterpolate, interpolate) {
  var u = uninterpolate(domain[0], domain[1]),
      i = interpolate(range[0], range[1]);
  return function(x) {
    return i(u(x));
  };
}

function polylinear(domain, range, uninterpolate, interpolate) {
  var k = Math.min(domain.length, range.length) - 1,
      u = new Array(k),
      i = new Array(k),
      j = -1;

  // Handle descending domains.
  if (domain[k] < domain[0]) {
    domain = domain.slice().reverse();
    range = range.slice().reverse();
  }

  while (++j < k) {
    u[j] = uninterpolate(domain[j], domain[j + 1]);
    i[j] = interpolate(range[j], range[j + 1]);
  }

  return function(x) {
    var j = d3Arrays.bisect(domain, x, 1, k) - 1;
    return i[j](u[j](x));
  };
}

function newLinear(domain, range, interpolate, clamp) {
  var output,
      input;

  function rescale() {
    var linear = Math.min(domain.length, range.length) > 2 ? polylinear : bilinear,
        uninterpolate = clamp ? uninterpolateClamp : uninterpolateNumber;
    output = linear(domain, range, uninterpolate, interpolate);
    input = linear(range, domain, uninterpolate, d3Interpolate.interpolateNumber);
    return scale;
  }

  function scale(x) {
    return output(x);
  }

  scale.invert = function(y) {
    return input(y);
  };

  scale.domain = function(x) {
    if (!arguments.length) return domain.slice();
    domain = x.map(Number);
    return rescale();
  };

  scale.range = function(x) {
    if (!arguments.length) return range.slice();
    range = x.slice();
    return rescale();
  };

  scale.rangeRound = function(x) {
    return scale.range(x).interpolate(d3Interpolate.interpolateRound);
  };

  scale.clamp = function(x) {
    if (!arguments.length) return clamp;
    clamp = !!x;
    return rescale();
  };

  scale.interpolate = function(x) {
    if (!arguments.length) return interpolate;
    interpolate = x;
    return rescale();
  };

  scale.ticks = function(count) {
    return ticks$1(domain, count);
  };

  scale.tickFormat = function(count, specifier) {
    return tickFormat$2(domain, count, specifier);
  };

  scale.nice = function(count) {
    domain = nice(domain, tickRange(domain, count)[2]);
    return rescale();
  };

  scale.copy = function() {
    return newLinear(domain, range, interpolate, clamp);
  };

  return rescale();
}

function rebind(scale, linear) {
  scale.range = function() {
    var x = linear.range.apply(linear, arguments);
    return x === linear ? scale : x;
  };

  scale.rangeRound = function() {
    var x = linear.rangeRound.apply(linear, arguments);
    return x === linear ? scale : x;
  };

  scale.clamp = function() {
    var x = linear.clamp.apply(linear, arguments);
    return x === linear ? scale : x;
  };

  scale.interpolate = function() {
    var x = linear.interpolate.apply(linear, arguments);
    return x === linear ? scale : x;
  };

  return scale;
};

function linear() {
  return newLinear([0, 1], [0, 1], d3Interpolate.interpolate, false);
};

function newDate(t) {
  return new Date(t);
}

function newTime(linear, timeInterval, tickFormat, format) {

  function scale(x) {
    return linear(x);
  }

  scale.invert = function(x) {
    return newDate(linear.invert(x));
  };

  scale.domain = function(x) {
    if (!arguments.length) return linear.domain().map(newDate);
    linear.domain(x);
    return scale;
  };

  function tickInterval(interval, start, stop, step) {
    if (interval == null) interval = 10;

    // If a desired tick count is specified, pick a reasonable tick interval
    // based on the extent of the domain and a rough estimate of tick size.
    // If a named interval such as "seconds" was specified, convert to the
    // corresponding time interval and optionally filter using the step.
    // Otherwise, assume interval is already a time interval and use it.
    switch (typeof interval) {
      case "number": interval = chooseTickInterval(start, stop, interval), step = interval[1], interval = interval[0]; break;
      case "string": step = step == null ? 1 : Math.floor(step); break;
      default: return interval;
    }

    return isFinite(step) && step > 0 ? timeInterval(interval, step) : null;
  }

  scale.ticks = function(interval, step) {
    var domain = linear.domain(),
        t0 = domain[0],
        t1 = domain[domain.length - 1],
        t;

    if (t1 < t0) t = t0, t0 = t1, t1 = t;

    return (interval = tickInterval(interval, t0, t1, step))
        ? interval.range(t0, t1 + 1) // inclusive stop
        : [];
  };

  scale.tickFormat = function(specifier) {
    return specifier == null ? tickFormat : format(specifier);
  };

  scale.nice = function(interval, step) {
    var domain = linear.domain(),
        i0 = 0,
        i1 = domain.length - 1,
        t0 = domain[i0],
        t1 = domain[i1],
        t;

    if (t1 < t0) {
      t = i0, i0 = i1, i1 = t;
      t = t0, t0 = t1, t1 = t;
    }

    if (interval = tickInterval(interval, t0, t1, step)) {
      domain[i0] = +interval.floor(t0);
      domain[i1] = +interval.ceil(t1);
      linear.domain(domain);
    }

    return scale;
  };

  scale.copy = function() {
    return newTime(linear.copy(), timeInterval, tickFormat, format);
  };

  return rebind(scale, linear);
};

var millisecondsPerSecond = 1000;
var millisecondsPerMinute = millisecondsPerSecond * 60;
var millisecondsPerHour = millisecondsPerMinute * 60;
var millisecondsPerDay = millisecondsPerHour * 24;
var millisecondsPerWeek = millisecondsPerDay * 7;
var millisecondsPerMonth = millisecondsPerDay * 30;
var millisecondsPerYear = millisecondsPerDay * 365;
var tickIntervals = [
  ["seconds",  1,      millisecondsPerSecond],
  ["seconds",  5,  5 * millisecondsPerSecond],
  ["seconds", 15, 15 * millisecondsPerSecond],
  ["seconds", 30, 30 * millisecondsPerSecond],
  ["minutes",  1,      millisecondsPerMinute],
  ["minutes",  5,  5 * millisecondsPerMinute],
  ["minutes", 15, 15 * millisecondsPerMinute],
  ["minutes", 30, 30 * millisecondsPerMinute],
  [  "hours",  1,      millisecondsPerHour  ],
  [  "hours",  3,  3 * millisecondsPerHour  ],
  [  "hours",  6,  6 * millisecondsPerHour  ],
  [  "hours", 12, 12 * millisecondsPerHour  ],
  [   "days",  1,      millisecondsPerDay   ],
  [   "days",  2,  2 * millisecondsPerDay   ],
  [  "weeks",  1,      millisecondsPerWeek  ],
  [ "months",  1,      millisecondsPerMonth ],
  [ "months",  3,  3 * millisecondsPerMonth ],
  [  "years",  1,      millisecondsPerYear  ]
];

var bisectTickIntervals = d3Arrays.bisector(function(method) {
  return method[2];
}).right;

function chooseTickInterval(start, stop, count) {
  var target = Math.abs(stop - start) / count,
      i = bisectTickIntervals(tickIntervals, target);
  return i === tickIntervals.length ? ["years", tickRange([start / millisecondsPerYear, stop / millisecondsPerYear], count)[2]]
      : i ? tickIntervals[target / tickIntervals[i - 1][2] < tickIntervals[i][2] / target ? i - 1 : i]
      : ["milliseconds", tickRange([start, stop], count)[2]];
}

var formatMillisecond = d3TimeFormat.format(".%L");
var formatSecond = d3TimeFormat.format(":%S");
var formatMinute = d3TimeFormat.format("%I:%M");
var formatHour = d3TimeFormat.format("%I %p");
var formatDay = d3TimeFormat.format("%a %d");
var formatWeek = d3TimeFormat.format("%b %d");
var formatMonth = d3TimeFormat.format("%B");
var formatYear = d3TimeFormat.format("%Y");
function tickFormat(date) {
  return (d3Time.second(date) < date ? formatMillisecond
      : d3Time.minute(date) < date ? formatSecond
      : d3Time.hour(date) < date ? formatMinute
      : d3Time.day(date) < date ? formatHour
      : d3Time.month(date) < date ? (d3Time.week(date) < date ? formatDay : formatWeek)
      : d3Time.year(date) < date ? formatMonth
      : formatYear)(date);
}

function millisecond(step) {
  return {
    range: function(start, stop) { return d3Arrays.range(Math.ceil(start / step) * step, stop, step).map(newDate); },
    floor: function(date) { return newDate(Math.floor(date / step) * step); },
    ceil: function(date) { return newDate(Math.ceil(date / step) * step); }
  };
};

function timeInterval(interval, step) {
  switch (interval) {
    case "milliseconds": return millisecond(step);
    case "seconds": return step > 1 ? d3Time.second.filter(function(d) { return d.getSeconds() % step === 0; }) : d3Time.second;
    case "minutes": return step > 1 ? d3Time.minute.filter(function(d) { return d.getMinutes() % step === 0; }) : d3Time.minute;
    case "hours": return step > 1 ? d3Time.hour.filter(function(d) { return d.getHours() % step === 0; }) : d3Time.hour;
    case "days": return step > 1 ? d3Time.day.filter(function(d) { return (d.getDate() - 1) % step === 0; }) : d3Time.day;
    case "weeks": return step > 1 ? d3Time.week.filter(function(d) { return d3Time.week.count(0, d) % step === 0; }) : d3Time.week;
    case "months": return step > 1 ? d3Time.month.filter(function(d) { return d.getMonth() % step === 0; }) : d3Time.month;
    case "years": return step > 1 ? d3Time.year.filter(function(d) { return d.getFullYear() % step === 0; }) : d3Time.year;
  }
}

function time() {
  return newTime(linear(), timeInterval, tickFormat, d3TimeFormat.format).domain([new Date(2000, 0, 1), new Date(2000, 0, 2)]);
};

var formatUTCMillisecond = d3TimeFormat.utcFormat(".%L");
var formatUTCSecond = d3TimeFormat.utcFormat(":%S");
var formatUTCMinute = d3TimeFormat.utcFormat("%I:%M");
var formatUTCHour = d3TimeFormat.utcFormat("%I %p");
var formatUTCDay = d3TimeFormat.utcFormat("%a %d");
var formatUTCWeek = d3TimeFormat.utcFormat("%b %d");
var formatUTCMonth = d3TimeFormat.utcFormat("%B");
var formatUTCYear = d3TimeFormat.utcFormat("%Y");
function tickFormat$1(date) {
  return (d3Time.utcSecond(date) < date ? formatUTCMillisecond
      : d3Time.utcMinute(date) < date ? formatUTCSecond
      : d3Time.utcHour(date) < date ? formatUTCMinute
      : d3Time.utcDay(date) < date ? formatUTCHour
      : d3Time.utcMonth(date) < date ? (d3Time.utcWeek(date) < date ? formatUTCDay : formatUTCWeek)
      : d3Time.utcYear(date) < date ? formatUTCMonth
      : formatUTCYear)(date);
}

function timeInterval$1(interval, step) {
  switch (interval) {
    case "milliseconds": return millisecond(step);
    case "seconds": return step > 1 ? d3Time.utcSecond.filter(function(d) { return d.getUTCSeconds() % step === 0; }) : d3Time.utcSecond;
    case "minutes": return step > 1 ? d3Time.utcMinute.filter(function(d) { return d.getUTCMinutes() % step === 0; }) : d3Time.utcMinute;
    case "hours": return step > 1 ? d3Time.utcHour.filter(function(d) { return d.getUTCHours() % step === 0; }) : d3Time.utcHour;
    case "days": return step > 1 ? d3Time.utcDay.filter(function(d) { return (d.getUTCDate() - 1) % step === 0; }) : d3Time.utcDay;
    case "weeks": return step > 1 ? d3Time.utcWeek.filter(function(d) { return d3Time.utcWeek.count(0, d) % step === 0; }) : d3Time.utcWeek;
    case "months": return step > 1 ? d3Time.utcMonth.filter(function(d) { return d.getUTCMonth() % step === 0; }) : d3Time.utcMonth;
    case "years": return step > 1 ? d3Time.utcYear.filter(function(d) { return d.getUTCFullYear() % step === 0; }) : d3Time.utcYear;
  }
}

function utcTime() {
  return newTime(linear(), timeInterval$1, tickFormat$1, d3TimeFormat.utcFormat).domain([Date.UTC(2000, 0, 1), Date.UTC(2000, 0, 2)]);
};

function newThreshold(domain, range, n) {

  function scale(x) {
    if (x <= x) return range[d3Arrays.bisect(domain, x, 0, n)];
  }

  scale.domain = function(x) {
    if (!arguments.length) return domain.slice();
    domain = x.slice(), n = Math.min(domain.length, range.length - 1);
    return scale;
  };

  scale.range = function(x) {
    if (!arguments.length) return range.slice();
    range = x.slice(), n = Math.min(domain.length, range.length - 1);
    return scale;
  };

  scale.invertExtent = function(y) {
    return y = range.indexOf(y), [domain[y - 1], domain[y]];
  };

  scale.copy = function() {
    return newThreshold(domain, range);
  };

  return scale;
};

function threshold() {
  return newThreshold([.5], [0, 1], 1);
};

function newPow(linear, exponent, domain) {

  function powp(x) {
    return x < 0 ? -Math.pow(-x, exponent) : Math.pow(x, exponent);
  }

  function powb(x) {
    return x < 0 ? -Math.pow(-x, 1 / exponent) : Math.pow(x, 1 / exponent);
  }

  function scale(x) {
    return linear(powp(x));
  }

  scale.invert = function(x) {
    return powb(linear.invert(x));
  };

  scale.exponent = function(x) {
    if (!arguments.length) return exponent;
    exponent = +x;
    return scale.domain(domain);
  };

  scale.domain = function(x) {
    if (!arguments.length) return domain.slice();
    domain = x.map(Number);
    linear.domain(domain.map(powp));
    return scale;
  };

  scale.ticks = function(count) {
    return ticks$1(domain, count);
  };

  scale.tickFormat = function(count, specifier) {
    return tickFormat$2(domain, count, specifier);
  };

  scale.nice = function(count) {
    return scale.domain(nice(domain, tickRange(domain, count)[2]));
  };

  scale.copy = function() {
    return newPow(linear.copy(), exponent, domain);
  };

  return rebind(scale, linear);
}

function sqrt() {
  return newPow(linear(), .5, [0, 1]);
};

function pow() {
  return newPow(linear(), 1, [0, 1]);
};

function rainbow() {
  return linear()
      .interpolate(d3Color.interpolateCubehelixLong)
      .domain([0, 0.5, 1.0])
      .range([d3Color.cubehelix(-100, 0.75, 0.35), d3Color.cubehelix(80, 1.50, 0.8), d3Color.cubehelix(260, 0.75, 0.35)]);
};

function newQuantize(x0, x1, range) {
  var kx, i;

  function scale(x) {
    return range[Math.max(0, Math.min(i, Math.floor(kx * (x - x0))))];
  }

  function rescale() {
    kx = range.length / (x1 - x0);
    i = range.length - 1;
    return scale;
  }

  scale.domain = function(x) {
    if (!arguments.length) return [x0, x1];
    x0 = +x[0];
    x1 = +x[x.length - 1];
    return rescale();
  };

  scale.range = function(x) {
    if (!arguments.length) return range.slice();
    range = x.slice();
    return rescale();
  };

  scale.invertExtent = function(y) {
    y = range.indexOf(y);
    y = y < 0 ? NaN : y / kx + x0;
    return [y, y + 1 / kx];
  };

  scale.copy = function() {
    return newQuantize(x0, x1, range); // copy on write
  };

  return rescale();
}

function quantize() {
  return newQuantize(0, 1, [0, 1]);
};

function newQuantile(domain, range) {
  var thresholds;

  function rescale() {
    var k = 0,
        q = range.length;
    thresholds = [];
    while (++k < q) thresholds[k - 1] = d3Arrays.quantile(domain, k / q);
    return scale;
  }

  function scale(x) {
    if (!isNaN(x = +x)) return range[d3Arrays.bisect(thresholds, x)];
  }

  scale.domain = function(x) {
    if (!arguments.length) return domain;
    domain = [];
    for (var i = 0, n = x.length, v; i < n; ++i) if (v = x[i], v != null && !isNaN(v = +v)) domain.push(v);
    domain.sort(d3Arrays.ascending);
    return rescale();
  };

  scale.range = function(x) {
    if (!arguments.length) return range.slice();
    range = x.slice();
    return rescale();
  };

  scale.quantiles = function() {
    return thresholds;
  };

  scale.invertExtent = function(y) {
    y = range.indexOf(y);
    return y < 0 ? [NaN, NaN] : [
      y > 0 ? thresholds[y - 1] : domain[0],
      y < thresholds.length ? thresholds[y] : domain[domain.length - 1]
    ];
  };

  scale.copy = function() {
    return newQuantile(domain, range); // copy on write!
  };

  return rescale();
}

function quantile() {
  return newQuantile([], []);
};

function steps(length, start, step) {
  var steps = new Array(length), i = -1;
  while (++i < length) steps[i] = start + step * i;
  return steps;
}

function newOrdinal(domain, ranger) {
  var index,
      range,
      rangeBand;

  function scale(x) {
    var k = x + "", i = index.get(k);
    if (!i) {
      if (ranger.t !== "range") return;
      index.set(k, i = domain.push(x));
    }
    return range[(i - 1) % range.length];
  }

  scale.domain = function(x) {
    if (!arguments.length) return domain.slice();
    domain = [];
    index = d3Arrays.map();
    var i = -1, n = x.length, xi, xk;
    while (++i < n) if (!index.has(xk = (xi = x[i]) + "")) index.set(xk, domain.push(xi));
    return scale[ranger.t].apply(scale, ranger.a);
  };

  scale.range = function(x) {
    if (!arguments.length) return range.slice();
    range = x.slice();
    rangeBand = 0;
    ranger = {t: "range", a: arguments};
    return scale;
  };

  scale.rangePoints = function(x, padding) {
    padding = arguments.length < 2 ? 0 : +padding;
    var start = +x[0],
        stop = +x[1],
        step = domain.length < 2 ? (start = (start + stop) / 2, 0) : (stop - start) / (domain.length - 1 + padding);
    range = steps(domain.length, start + step * padding / 2, step);
    rangeBand = 0;
    ranger = {t: "rangePoints", a: arguments};
    return scale;
  };

  scale.rangeRoundPoints = function(x, padding) {
    padding = arguments.length < 2 ? 0 : +padding;
    var start = +x[0],
        stop = +x[1],
        step = domain.length < 2 ? (start = stop = Math.round((start + stop) / 2), 0) : (stop - start) / (domain.length - 1 + padding) | 0; // bitwise floor for symmetry
    range = steps(domain.length, start + Math.round(step * padding / 2 + (stop - start - (domain.length - 1 + padding) * step) / 2), step);
    rangeBand = 0;
    ranger = {t: "rangeRoundPoints", a: arguments};
    return scale;
  };

  scale.rangeBands = function(x, padding, outerPadding) {
    padding = arguments.length < 2 ? 0 : +padding;
    outerPadding = arguments.length < 3 ? padding : +outerPadding;
    var reverse = +x[1] < +x[0],
        start = +x[reverse - 0],
        stop = +x[1 - reverse],
        step = (stop - start) / (domain.length - padding + 2 * outerPadding);
    range = steps(domain.length, start + step * outerPadding, step);
    if (reverse) range.reverse();
    rangeBand = step * (1 - padding);
    ranger = {t: "rangeBands", a: arguments};
    return scale;
  };

  scale.rangeRoundBands = function(x, padding, outerPadding) {
    padding = arguments.length < 2 ? 0 : +padding;
    outerPadding = arguments.length < 3 ? padding : +outerPadding;
    var reverse = +x[1] < +x[0],
        start = +x[reverse - 0],
        stop = +x[1 - reverse],
        step = Math.floor((stop - start) / (domain.length - padding + 2 * outerPadding));
    range = steps(domain.length, start + Math.round((stop - start - (domain.length - padding) * step) / 2), step);
    if (reverse) range.reverse();
    rangeBand = Math.round(step * (1 - padding));
    ranger = {t: "rangeRoundBands", a: arguments};
    return scale;
  };

  scale.rangeBand = function() {
    return rangeBand;
  };

  scale.rangeExtent = function() {
    var t = ranger.a[0], start = t[0], stop = t[t.length - 1];
    if (stop < start) t = stop, stop = start, start = t;
    return [start, stop];
  };

  scale.copy = function() {
    return newOrdinal(domain, ranger);
  };

  return scale.domain(domain);
}

function ordinal() {
  return newOrdinal([], {t: "range", a: [[]]});
};

var tickFormat10 = d3Format.format(".0e");
var tickFormatOther = d3Format.format(",");
function newLog(linear, base, domain) {

  function log(x) {
    return (domain[0] < 0 ? -Math.log(x > 0 ? 0 : -x) : Math.log(x < 0 ? 0 : x)) / Math.log(base);
  }

  function pow(x) {
    return domain[0] < 0 ? -Math.pow(base, -x) : Math.pow(base, x);
  }

  function scale(x) {
    return linear(log(x));
  }

  scale.invert = function(x) {
    return pow(linear.invert(x));
  };

  scale.base = function(x) {
    if (!arguments.length) return base;
    base = +x;
    return scale.domain(domain);
  };

  scale.domain = function(x) {
    if (!arguments.length) return domain.slice();
    domain = x.map(Number);
    linear.domain(domain.map(log));
    return scale;
  };

  scale.nice = function() {
    var x = nice(linear.domain(), 1);
    linear.domain(x);
    domain = x.map(pow);
    return scale;
  };

  scale.ticks = function() {
    var u = domain[0],
        v = domain[domain.length - 1];
    if (v < u) i = u, u = v, v = i;
    var i = Math.floor(log(u)),
        j = Math.ceil(log(v)),
        k,
        t,
        n = base % 1 ? 2 : base,
        ticks = [];

    if (isFinite(j - i)) {
      if (u > 0) {
        for (--j, k = 1; k < n; ++k) if ((t = pow(i) * k) < u) continue; else ticks.push(t);
        while (++i < j) for (k = 1; k < n; ++k) ticks.push(pow(i) * k);
        for (k = 1; k < n; ++k) if ((t = pow(i) * k) > v) break; else ticks.push(t);
      } else {
        for (++i, k = n - 1; k >= 1; --k) if ((t = pow(i) * k) < u) continue; else ticks.push(t);
        while (++i < j) for (k = n - 1; k >= 1; --k) ticks.push(pow(i) * k);
        for (k = n - 1; k >= 1; --k) if ((t = pow(i) * k) > v) break; else ticks.push(t);
      }
    }

    return ticks;
  };

  scale.tickFormat = function(count, specifier) {
    if (specifier == null) specifier = base === 10 ? tickFormat10 : tickFormatOther;
    else if (typeof specifier !== "function") specifier = d3Format.format(specifier);
    if (count == null) return specifier;
    var k = Math.min(base, scale.ticks().length / count),
        f = domain[0] > 0 ? (e = 1e-12, Math.ceil) : (e = -1e-12, Math.floor),
        e;
    return function(d) {
      return pow(f(log(d) + e)) / d >= k ? specifier(d) : "";
    };
  };

  scale.copy = function() {
    return newLog(linear.copy(), base, domain);
  };

  return rebind(scale, linear);
}

function log() {
  return newLog(linear(), 10, [1, 10]);
};

function newIdentity(domain) {

  function scale(x) {
    return +x;
  }

  scale.invert = scale;

  scale.domain = scale.range = function(x) {
    if (!arguments.length) return domain.slice();
    domain = x.map(Number);
    return scale;
  };

  scale.ticks = function(count) {
    return ticks$1(domain, count);
  };

  scale.tickFormat = function(count, specifier) {
    return tickFormat$2(domain, count, specifier);
  };

  scale.copy = function() {
    return newIdentity(domain);
  };

  return scale;
}

function identity() {
  return newIdentity([0, 1]);
};

function cubehelix() {
  return linear()
      .interpolate(d3Color.interpolateCubehelixLong)
      .range([d3Color.cubehelix(300, 0.5, 0.0), d3Color.cubehelix(-240, 0.5, 1.0)]);
};

function category20c() {
  return ordinal().range([
    "#3182bd", "#6baed6", "#9ecae1", "#c6dbef",
    "#e6550d", "#fd8d3c", "#fdae6b", "#fdd0a2",
    "#31a354", "#74c476", "#a1d99b", "#c7e9c0",
    "#756bb1", "#9e9ac8", "#bcbddc", "#dadaeb",
    "#636363", "#969696", "#bdbdbd", "#d9d9d9"
  ]);
};

function category20b() {
  return ordinal().range([
    "#393b79", "#5254a3", "#6b6ecf", "#9c9ede",
    "#637939", "#8ca252", "#b5cf6b", "#cedb9c",
    "#8c6d31", "#bd9e39", "#e7ba52", "#e7cb94",
    "#843c39", "#ad494a", "#d6616b", "#e7969c",
    "#7b4173", "#a55194", "#ce6dbd", "#de9ed6"
  ]);
};

function category20() {
  return ordinal().range([
    "#1f77b4", "#aec7e8",
    "#ff7f0e", "#ffbb78",
    "#2ca02c", "#98df8a",
    "#d62728", "#ff9896",
    "#9467bd", "#c5b0d5",
    "#8c564b", "#c49c94",
    "#e377c2", "#f7b6d2",
    "#7f7f7f", "#c7c7c7",
    "#bcbd22", "#dbdb8d",
    "#17becf", "#9edae5"
  ]);
};

function category10() {
  return ordinal().range([
    "#1f77b4",
    "#ff7f0e",
    "#2ca02c",
    "#d62728",
    "#9467bd",
    "#8c564b",
    "#e377c2",
    "#7f7f7f",
    "#bcbd22",
    "#17becf"
  ]);
};

exports.category10 = category10;
exports.category20 = category20;
exports.category20b = category20b;
exports.category20c = category20c;
exports.cubehelix = cubehelix;
exports.identity = identity;
exports.linear = linear;
exports.log = log;
exports.ordinal = ordinal;
exports.pow = pow;
exports.quantile = quantile;
exports.quantize = quantize;
exports.rainbow = rainbow;
exports.sqrt = sqrt;
exports.threshold = threshold;
exports.time = time;
exports.utcTime = utcTime;